﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace VezZLega
{
    public partial class Display : Form
    {
        private Tower tower;

        public Display()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            openFileDialog1.ShowDialog();
            textBox1.Text = openFileDialog1.FileName;
        }

        private void button2_Click(object sender, EventArgs e)
        {
            tower = new Tower(textBox1.Text, false);

            listBox1.Items.Clear();

            foreach(Block b in tower.Material.Keys)
            {
                listBox1.Items.Add("Kostka: " + b.ToString() + " Počet: " + tower.Material[b]);
            }

            numericUpDown1.Maximum = tower.Levels.Count;
            numericUpDown1.Value = 1;
            DisplayLevel(1);

            pictureBox2.Image = DrawLevel(1);
            pictureBox3.Image = DrawLevel(tower.Levels.Count);

            pictureBox4.Image = DrawSide('n');
            pictureBox5.Image = DrawSide('s'); 
            pictureBox6.Image = DrawSide('e');
            pictureBox7.Image = DrawSide('w');
        }

        private void DisplayLevel(int level)
        {
            pictureBox1.Image = DrawLevel(level);
        }

        private Bitmap DrawLevel(int level)
        {
            PlacedBlock[] blocks = tower.Levels[level - 1].ToArray();

            Bitmap bmp = new Bitmap(180, 180);
            Graphics g = Graphics.FromImage(bmp);

            float xUnit = bmp.Width / (float)tower.BaseWidth;
            float yUnit = bmp.Height / (float)tower.BaseHeight;

            Pen p = new Pen(Color.Black, 2);

            foreach (PlacedBlock pb in blocks)
            {
                SolidBrush b = new SolidBrush(pb.Block.Color);

                g.FillRectangle(b, xUnit * (pb.X - 1), yUnit * (pb.Y - 1), xUnit * pb.Block.Width, yUnit * pb.Block.Height);

                g.DrawRectangle(p, xUnit * (pb.X - 1), yUnit * (pb.Y - 1), xUnit * pb.Block.Width, yUnit * pb.Block.Height);
            }

            return bmp;
        }

        private Bitmap DrawSide(char side)
        {
            Bitmap final = new Bitmap(100, 260);
            Graphics g = Graphics.FromImage(final);

            float zUnit = 260 / (float)tower.Levels.Count;
            float xUnit = 180 / (float)tower.BaseWidth;
            float yUnit = 180 / (float)tower.BaseHeight;

            Bitmap[] levels = new Bitmap[tower.Levels.Count];

            for (int i = 1; i <= levels.Length; i++)
            {
                levels[i-1] = DrawLevel(i);
            }

            switch (side)
            {
                case 'n':
                    for (int i = 0; i < levels.Length; i++)
                    {
                        g.DrawImage(levels[i], new RectangleF(0, final.Height - zUnit * (i+1), 100, zUnit), new RectangleF(0, 0, 180, yUnit), GraphicsUnit.Pixel);
                    }
                    final.RotateFlip(RotateFlipType.RotateNoneFlipX);
                    break;
                case 's':
                    for (int i = 0; i < levels.Length; i++)
                    {
                        g.DrawImage(levels[i], new RectangleF(0, final.Height - zUnit * (i + 1), 100, zUnit), new RectangleF(0, 180-yUnit, 180, yUnit), GraphicsUnit.Pixel);
                    }
                    break;
                case 'e':
                    for (int i = 0; i < levels.Length; i++)
                    {
                        levels[i].RotateFlip(RotateFlipType.Rotate90FlipNone);
                        g.DrawImage(levels[i], new RectangleF(0, final.Height - zUnit * (i + 1), 100, zUnit), new RectangleF(0, 180 - yUnit, 180, yUnit), GraphicsUnit.Pixel);
                    }
                    break;
                case 'w':
                    for (int i = 0; i < levels.Length; i++)
                    {
                        levels[i].RotateFlip(RotateFlipType.Rotate270FlipNone);
                        g.DrawImage(levels[i], new RectangleF(0, final.Height - zUnit * (i + 1), 100, zUnit), new RectangleF(0, 180 - yUnit, 180, yUnit), GraphicsUnit.Pixel);
                    }
                    break;
            }

            return final;
        }

        private void numericUpDown1_ValueChanged(object sender, EventArgs e)
        {
            DisplayLevel((int)numericUpDown1.Value);
        }

        private void pictureBox6_Click(object sender, EventArgs e)
        {

        }
    }
}
