﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Text.RegularExpressions;

namespace VezZLega
{
    public class Tower
    {
        private int baseWidth;
        private int baseHeight;
        private string type;
        private Dictionary<Block, int> material;
        private List<List<PlacedBlock>> levels;

        public int BaseWidth
        {
            get { return baseWidth; }
            set { baseWidth = value; }
        }

        public int BaseHeight
        {
            get { return baseHeight; }
            set { baseHeight = value; }
        }

        public string Type
        {
            get { return type; }
            set { type = value; }
        }

        public List<List<PlacedBlock>> Levels
        {
            get { return levels; }
            set { levels = value; }
        }

        public Dictionary<Block, int> Material
        {
            get { return material; }
            set { material = value; }
        }

        public Tower (string inputFile, bool shouldBuild)
        {
            string[] lines = File.ReadAllLines(inputFile);
            Regex towerBase = new Regex("([0-9]{1,2})[ ]+([0-9]{1,2})[ ]+([PDB]{1,2})");

            var towerBaseMatch = towerBase.Match(lines[0]);
            baseWidth = int.Parse(towerBaseMatch.Groups[1].Value);
            baseHeight = int.Parse(towerBaseMatch.Groups[2].Value);
            type = towerBaseMatch.Groups[3].Value;

            if (shouldBuild)
            {
                Regex blockRX = new Regex("([ARGBY]{1})[ ]+([0-9]{1,2})[xX]([0-9]{1,2})[ ]+([0-9]+)");
                material = new Dictionary<Block, int>();

                for (int i = 1; i < lines.Length; i++)
                {
                    var blockMatch = blockRX.Match(lines[i]);

                    material.Add(new Block(int.Parse(blockMatch.Groups[2].Value), int.Parse(blockMatch.Groups[3].Value), Block.ParseColor(blockMatch.Groups[1].Value)), int.Parse(blockMatch.Groups[4].Value));
                }

                if (type[0] == 'P')
                    BuildFull(type.Length > 1 && type[1] == 'B');
                else if (type[0] == 'D')
                    BuildEmpty(type.Length > 1 && type[1] == 'B');
            }

            else
            {
                Regex blockRX = new Regex("([0-9]+)[ ]+([0-9]{1,2})[ ]+([0-9]{1,2})[ ]+([ARGBY]{1})[ ]+([0-9]{1,2})[xX]([0-9]{1,2})");
                material = new Dictionary<Block, int>();
                levels = new List<List<PlacedBlock>>();

                for (int i = 1; i < lines.Length; i++)
                {
                    var blockMatch = blockRX.Match(lines[i]);

                    if (blockMatch.Success)
                    {
                        Block b = new Block(int.Parse(blockMatch.Groups[5].Value), int.Parse(blockMatch.Groups[6].Value), Block.ParseColor(blockMatch.Groups[4].Value));
                        int level = int.Parse(blockMatch.Groups[1].Value);
                        int x = int.Parse(blockMatch.Groups[2].Value);
                        int y = int.Parse(blockMatch.Groups[3].Value);

                        if (level-1 >= levels.Count)
                        {
                            levels.Add(new List<PlacedBlock>());
                        }

                        levels[level-1].Add(new PlacedBlock(b, x, y));

                        if (b.Width > b.Height)
                        {
                            int z = b.Height;
                            b.Height = b.Width;
                            b.Width = z;
                        }

                        if (material.ContainsKey(b))
                        {
                            material[b]++;
                        }
                        else
                        {
                            material.Add(b, 1);
                        }
                    }
                }
            }
        }

        private void BuildFull(bool colors)
        {
            levels = new List<List<PlacedBlock>>();
        }

        private void BuildEmpty(bool colors)
        {
            levels = new List<List<PlacedBlock>>();

            if (!colors)
            {

            }
        }

        public void Save(string targetFile)
        {
            using (StreamWriter sw = new StreamWriter(targetFile, false))
            {
                sw.WriteLine(baseWidth + "  " + baseHeight + "  " + type);

                for (int i = 0; i < levels.Count; i++)
                {
                    foreach(PlacedBlock block in levels[i])
                    {
                        sw.WriteLine(i + "  " + block.X + "  " + block.Y + "  " + block.Block.Color.Name[0] + "  " + block.Block.Width + "x" + block.Block.Height);
                    }
                }
            }
        }
    }
}
